@@ -1,6 +1,52 @@
-from functools import partial
+# BSD 2-Clause License
+#
+# Copyright (c) 2021, Julien Phalip
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
+#
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-__all__ = ['TicSerial', 'TicI2C', 'TicUSB', 'TIC_T825', 'TIC_T834', 'TIC_T500', 'TIC_N825', 'TIC_T249', 'TIC_36v4']
+__all__ = [
+    'TicSerial', 'TicI2C', 'TicUSB', 'SMBus2Backend', 'MachineI2CBackend',
+    'TIC_T825', 'TIC_T834', 'TIC_T500', 'TIC_N825', 'TIC_T249', 'TIC_36v4'
+]
+
+import sys
+
+if sys.implementation.name == 'micropython':
+    def partial(function, *args):
+        """
+        Substitute to Python's `functools.partial`
+        """
+        def _partial(*extra_args):
+            extra_args = extra_args[1:]
+            return function(*(args + extra_args))
+        return _partial
+else:
+    from functools import partial
+
+try:
+    from machine import I2C as machine_i2c
+except ImportError:
+    machine_i2c = None
 
 try:
     from smbus2 import i2c_msg
@@ -18,6 +64,45 @@ except ImportError:
     usb_core = None
 
 
+class MachineI2CBackend(object):
+    """
+    I2C backend that acts as a wrapper around the `machine.I2C` class.
+    """
+
+    def __init__(self, i2c, address):
+        if machine_i2c is None:
+            raise Exception("Missing dependency: machine.I2C (Micropython)")
+        self.i2c = i2c
+        self.address = address
+
+    def read(self, length):
+        return self.i2c.readfrom(self.address, length)
+
+    def write(self, serialized):
+        self.i2c.writeto(self.address, serialized)
+
+
+class SMBus2Backend(object):
+    """
+    I2C Backend that uses the smbus2 library.
+    """
+
+    def __init__(self, bus, address):
+        if i2c_msg is None:
+            raise Exception("Missing dependency: smbus2")
+        self.address = address
+        self.bus = bus
+
+    def read(self, length):
+        read = i2c_msg.read(self.address, length)
+        self.bus.i2c_rdwr(read)
+        return read.__bytes__()[0: length]
+
+    def write(self, serialized):
+        write = i2c_msg.write(self.address, serialized)
+        self.bus.i2c_rdwr(write)
+
+
 # Output formatting functions ---------------------------------------------------
 
 def boolean(bit_index, value):
@@ -354,11 +439,17 @@ class TicBase(object):
         raise NotImplementedError
 
     def _define_commands(self):
+        """
+        Defines methods for all Tic commands.
+        """
         for command in COMMANDS:
             name, code, format = command
             setattr(self.__class__, name, partial(self._send_command, code, format))
 
     def _define_variables(self):
+        """
+        Defines methods for all Tic variables.
+        """
         self.variables = {}
         for variable in VARIABLES:
             name, offset, length, format_response = variable
@@ -369,7 +460,7 @@ class TicBase(object):
 
     def get_variables(self):
         """
-        Returns all of the Tic's variables and their values.
+        Returns all Tic variables and their values.
         """
         result = {}
         for variable in VARIABLES:
@@ -400,8 +491,6 @@ class TicSerial(TicBase):
     """
 
     def __init__(self, port, device_number=None, crc_for_commands=False, crc_for_responses=False):
-        if serial is None:
-            raise Exception("Missing dependency: pyserial")
         self.port = port
         self.device_number = device_number
         self.crc_for_commands = crc_for_commands
@@ -491,11 +580,8 @@ class TicI2C(TicBase):
     Reference: https://www.pololu.com/docs/0J71/10
     """
 
-    def __init__(self, bus, address):
-        if i2c_msg is None:
-            raise Exception("Missing dependency: smbus2")
-        self.bus = bus
-        self.address = address
+    def __init__(self, backend):
+        self.backend = backend
         super().__init__()
 
     def _send_command(self, command_code, format, value=None):
@@ -516,14 +602,14 @@ class TicI2C(TicBase):
         elif format == BLOCK_READ:
             serialized += value
         # Write command to the bus
-        self._write(serialized)
+        self.backend.write(serialized)
 
     def _block_read(self, command_code, offset, length, format_response=None):
         """
         Returns the value of the specified variable or setting from the Tic.
         """
         self._send_command(command_code, BLOCK_READ, bytes([offset, length]))
-        result = self._read(length)
+        result = self.backend.read(length)
         if len(result) != length:
             raise RuntimeError("Expected to read {} bytes, got {}.".format(length, len(result)))
         if format_response is None:
@@ -531,15 +617,6 @@ class TicI2C(TicBase):
         else:
             return format_response(result)
 
-    def _read(self, length):
-        read = i2c_msg.read(self.address, length)
-        self.bus.i2c_rdwr(read)
-        return read.__bytes__()[0: length]
-
-    def _write(self, serialized):
-        write = i2c_msg.write(self.address, serialized)
-        self.bus.i2c_rdwr(write)
-
 
 class TicUSB(TicBase):
     """
